@@ -25,6 +25,8 @@
 
 from collections import Counter
 import itertools
+import os
+import re
 import sys
 
 import numpy as np
@@ -34,6 +36,8 @@ import matplotlib.pyplot as plt
 from matplotlib.lines import Line2D
 from matplotlib.patches import Patch
 
+import scanpy as sc
+
 import seaborn as sns
 
 from anndata import AnnData
@@ -46,11 +50,80 @@ from abstar.core.germline import get_germlines
 
 # ===========================
 
-#         FEATURES
+#      QUALITY CONTROL
 
 # ===========================
 
 
+def qc_metrics(adata, fig_dir=None, fig_prefix=None):
+    if 'ig' not in adata.var:
+        pattern = re.compile('IG[HKL][VDJ][1-9].+|TR[ABDG][VDJ][1-9]')
+        adata.var['ig'] = [False if re.match(pattern, a) is None else True for a in adata.var.index]
+    if 'mt' not in adata.var:
+        adata.var['mt'] = adata.var_names.str.startswith('MT-')
+    sc.pp.calculate_qc_metrics(adata, qc_vars=['ig', 'mt'],
+                               percent_top=None, log1p=False, inplace=True)
+
+    # plot Ig
+    g = sns.JointGrid(data=adata.obs, x='total_counts', y='pct_counts_ig')
+    g.plot_joint(sns.scatterplot, s=10, fc='grey', linewidth=0)
+    g.plot_marginals(sns.kdeplot, shade=True, color='#404040')
+    g.ax_joint.set_xlabel('total counts', fontsize=16)
+    g.ax_joint.set_ylabel('immunoglobulin counts (%)', fontsize=16)
+    g.ax_joint.tick_params(axis='both', labelsize=13)
+    if fig_dir is not None:
+        plt.tight_layout()
+        if fig_prefix is not None:
+            fig_name = f'{fig_prefix}_pct-counts-ig.pdf'
+        else:
+            fig_name = 'pct_counts_ig.pdf'
+        plt.savefig(os.path.join(fig_dir, fig_name))
+    else:
+        plt.show()
+
+    # plot mito
+    g = sns.JointGrid(data=adata.obs, x='total_counts', y='pct_counts_mt')
+    g.plot_joint(sns.scatterplot, s=10, fc='grey', linewidth=0)
+    g.plot_marginals(sns.kdeplot, shade=True, color='#404040')
+    g.ax_joint.set_xlabel('total counts', fontsize=16)
+    g.ax_joint.set_ylabel('mitochondrial counts (%)', fontsize=16)
+    g.ax_joint.tick_params(axis='both', labelsize=13)
+    if fig_dir is not None:
+        plt.tight_layout()
+        if fig_prefix is not None:
+            fig_name = f'{fig_prefix}_pct-counts-mt.pdf'
+        else:
+            fig_name = 'pct_counts_mt.pdf'
+        plt.savefig(os.path.join(fig_dir, fig_name))
+    else:
+        plt.show()
+
+    # plot N genes by counts
+    g = sns.JointGrid(data=adata.obs, x='total_counts', y='n_genes_by_counts')
+    g.plot_joint(sns.scatterplot, s=10, fc='grey', linewidth=0)
+    g.plot_marginals(sns.kdeplot, shade=True, color='#404040')
+    g.ax_joint.set_xlabel('total counts', fontsize=16)
+    g.ax_joint.set_ylabel('number of genes', fontsize=16)
+    g.ax_joint.tick_params(axis='both', labelsize=13)
+    if fig_dir is not None:
+        plt.tight_layout()
+        if fig_prefix is not None:
+            fig_name = f'{fig_prefix}_n-genes-by-counts.pdf'
+        else:
+            fig_name = 'n_genes_by_counts.pdf'
+        plt.savefig(os.path.join(fig_dir, fig_name))
+    else:
+        plt.show()
+
+
+
+
+# ===========================
+
+#         FEATURES
+
+# ===========================
+
 
 def feature_kde(data, x, y, hue=None, hue_order=None, colors=None, thresh=0.1,
                 show_scatter=True, scatter_size=5, scatter_alpha=0.2,
@@ -634,6 +707,13 @@ def feature_ridge(data, features, colors=None, rename=None,
 
 
 
+# ===========================
+
+#           VDJ
+
+# ===========================
+
+
 def germline_use_barplot(adata, fig_file=None, gene='V', species='human', chain='heavy',
                          show=False,  pairs_only=False, normalize=True, figfile=None):
     '''
